from time import time
from typing import Optional, Any
from .base import AbstractStore

class InMemoryStore(AbstractStore):
    """
    In-memory implementation of the store using a dictionary.
    """

    def __init__(self):
        self._store = {}  # key -> (value, expiration_timestamp)

    async def get(self, key: str) -> Optional[Any]:
        if key not in self._store:
            return None

        value, expiration = self._store[key]
        current_time = time()

        if expiration is not None and current_time > expiration:
            del self._store[key]
            return None

        return value

    async def set(self, key: str, value: Any, ttl: int) -> bool:
        current_time = time()
        expiration = current_time + ttl if ttl > 0 else None
        self._store[key] = (value, expiration)
        return True

    async def delete(self, key: str) -> bool:
        if key in self._store:
            del self._store[key]
            return True
        return False


from abc import ABC, abstractmethod
from typing import Optional, Any

class AbstractStore(ABC):
    """Abstract base class for key-value stores (Async)."""

    @abstractmethod
    async def get(self, key: str) -> Optional[Any]:
        """Get a value from the store."""
        pass

    @abstractmethod
    async def set(self, key: str, value: Any, ttl: int) -> bool:
        """Set a value in the store with TTL."""
        pass

    @abstractmethod
    async def delete(self, key: str) -> bool:
        """Delete a value from the store."""
        pass




from typing import Optional, Any
import redis.asyncio as redis
from .base import AbstractStore

class RedisStore(AbstractStore):
    def __init__(self, client: redis.Redis):
        self.client = client

    async def get(self, key: str) -> Optional[Any]:
        key_type = await self.client.type(key)

        if key_type == b'hash':
            return await self.client.hgetall(key)
        elif key_type == b'string':
            return await self.client.get(key)
        else:
            return None

    async def set(self, key: str, value: Any, ttl: int) -> bool:
        if isinstance(value, dict):
            key_type = await self.client.type(key)
            if key_type != b'hash' and key_type != b'none':
                await self.client.delete(key)
            await self.client.hset(key, mapping=value)
            return await self.client.expire(key, ttl)
        else:
            key_type = await self.client.type(key)
            if key_type != b'string' and key_type != b'none':
                await self.client.delete(key)
            return await self._set_non_dict(key, value, ttl)

    async def _set_non_dict(self, key: str, value: Any, ttl: int) -> bool:
        if not isinstance(value, (str, bytes, int, float)):
            value = str(value)
        return await self.client.set(key, value, ex=ttl)

    async def delete(self, key: str) -> bool:
        result = await self.client.delete(key)
        return result > 0